## -*-cperl-*-
## Filename: UCS/R/Statistics.pm
## Modified: Wed Aug 17 23:50:26 2005 (severt)   
##   Author: Stefan Evert
##  Purpose: communicate with R backend through Statistics::R library

package UCS::R::Statistics;

use UCS;
use UCS::File;

use Statistics::R;

use base Exporter;
@EXPORT = qw(Start Stop Exec LoadVector DumpVector);

$R = undef;  # object for R bridge

# start R server process
sub Start ( ) {
  return 1 if $R and $R->is_started;     # already running

  UCS::Status "Spawning R server ... ";
  $R = new Statistics::R "r_bin" => $UCS::Rbin
    unless $R;
  $R->startR;
 
  UCS::Die "startup failed.\n"
    unless $R->is_started;
 
  _exec("options(digits=22, width=10)"); # maximal accuracy for floating-point value, print only 1 value per line (usually)
  _exec("suppressWarnings(source('${UCS::R_BaseDir}/R/lib/ucs.R'))"); # load UCS libraries
  _exec("ucs.library('sfunc')");
  
  UCS::Status "R has been set up successfully.\n";
  return 1;
}

# stop R server process
sub Stop ( ) {
  return 1 unless $R->is_started;     # no server running
  
  UCS::Status "Stopping R server ... ";
  $R->stopR;

  UCS::Status "ok\n";
  return 1;
}

# internal function to execute command, check for errors, and return result lines
sub _exec {
  UCS::Die "Internal error in UCS::R::Statistics: _exec() called, but R server is not running\n"
    unless $R and $R->is_started;
  UCS::Die "Internal error in UCS::R::Statistics: _exec() called with no/multiple arguments\n"
    if @_ != 1;
    
  my $cmd = shift;
  $R->send($cmd.'; cat("\n-::-EOL-::-\n")'); # check for success (trick inherited from CQP)
  my @lines = split /\r?\n/, $R->read;
  UCS::Die "UCS::R::Statistics: failed to execute command:\n\t$cmd\n"
    unless @lines >= 1 and (pop @lines) eq "-::-EOL-::-";
  pop @lines
    if @lines >= 1 and $lines[-1] eq "";
  return @lines;
}

# @results = UCS::R::Exec("$cmd");
# execute R command and return result (which is usually a vector)
sub Exec ( $ ) {
  my $cmd = shift;
  Start() unless $R and $R->is_started;                # auto-start R server when necessary
  $cmd =~ tr[\n][ ]s;                           # command should be on single line;
  _exec(".value <- $cmd"); # somewhat complicated because of limitations of Statistics::R
  my @output = _exec('print(.value)');
  
  my @result = ();  # need to parse vector result as in Expect.pm
  foreach (@output) {
    /^\s*\[([0-9]+)\]\s*(.*?)$/
      or UCS::Die "UCS::R::Statistics: can't parse R output ``$_''";
    my $idx = $1;
    my @items = split " ", $2;
    while (@items) {
      my $exp_idx = @result + 1;
      UCS::Die "UCS::R::Statistics: indexing problem with R output (expected $exp_idx, got $idx)"
        unless $exp_idx == $idx;
      push @result, shift @items;
      $idx++;
    }
  }
 
  return (wantarray) ? @result : $result[0];
}

# UCS::R::LoadVector($varname, \@data);
# load numeric vector into R through temporary file, using the scan() function
sub LoadVector ( $$ ) {
  my $var = shift;
  my $data = shift;
  my $l = scalar @$data;
  if ($l == 0) {
    Exec("$var <<- numeric(0)"); # ensure that we're writing to global namespace
  }
  else {
    my $tf = new UCS::File::Temp;
    for my $i (0 .. $l - 1) {
      $tf->write($data->[$i], "\n");
    }
    $tf->finish;
    my $filename = $tf->name;
    Exec("$var <<- scan(file=\"$filename\", quiet=TRUE, sep=\"\\n\", n=$l)");
    $tf->close;
  }
  my $l2 = Exec("length($var)");
  UCS::Die "UCS::R::LoadVector(): load failed (sent $l entries, received $l2)"
      unless $l2 == $l;
}

# $arrayref = UCS::R::DumpVector($varname);
# read numeric vector from R through temporary file, using the write() function;
# the data are returned as an anonymous array reference for efficiency reasons
sub DumpVector ( $ ) {
  my $var = shift;
  my @data = ();
  my $l = Exec("length($var)");
  if ($l == 0) {
    return [];
  }
  else {
    my $tf = new UCS::File::Temp;
    $tf->finish;
    my $filename = $tf->name;
    Exec("write($var, file=\"$filename\", ncolumns=1)");
    $data[$l-1] = 0;                            # pre-allocate array
    my $size = 0;
    while (my $line = $tf->read) {
      chomp $line;
      $data[$size] = $line;
      $size++;
    }
    $tf->close;
    UCS::Die "UCS::R::DumpVector(): dump failed (expected $l entries, got $size)"
        unless $size == $l;
    return \@data;
  }
}

1;

__END__


=head1 NAME

UCS::R::Statistics - R backend using the Statistics::R bridge

=head1 SYNOPSIS

  use UCS::R::Statistics;
  ## exports Start(), Stop() and Exec() functions into current namespace
  ## as well as LoadVector() and DumpVector()


=head1 DESCRIPTION

This module should not be used directly.  Import B<UCS::R> instead, which
will automatically load an available backend (B<UCS::R::RSPerl>, B<UCS::R::Statistics>,
or B<UCS::R::Expect>) as determined during the installation process.

=head1 LIMITATIONS

This module starts an R process in the background and communicates with it
interactively through a B<Statistics::R> bridge.  This approach has several
disadvantages:

=over 4

=item *

Invoking R commands, waiting for output from the R backend, and parsing that
output is an extremely slow process with B<Statistics::R>.  In some situations,
only a few hundred function invocations per second are achieved.

=item *

The return value of a function call has to be printed by R, then the resulting
output has to be parsed by Perl.  This interfacing method is rather frail and
currently supports only numeric vectors as return values.

=item *

The interface is extremely inefficient for exchanging large amounts of data
between Perl and R.  It may hang if the input/output buffers used by B<Expect>
overflow.  Use the B<LoadVector> and B<DumpVector> functions to pass large
numeric vectors to R and back.

=back

Because of these limitations, it is recommended that you install and
use the B<RSPerl> interface (available from L<http://www.omegahat.org/>) on
Unix platforms, or at least the somewhat more efficient B<Expect>
package.  The UCS installation script will automatically determine the best
available backend.  See F<doc/install.txt> for more information and
installation tips.

=head1 COPYRIGHT

Copyright (C) 2004-2005 Stefan Evert.

This software is provided AS IS and the author makes no warranty as to
its use and performance. You may use the software, redistribute and
modify it under the same terms as Perl itself.

=cut
